@@ -4,6 +4,7 @@ use core::fmt::{self, Debug, Formatter};
 use core::intrinsics;
 use core::iter::{FusedIterator, TrustedLen};
 use core::marker::{PhantomData, Send, Sync};
+use core::mem::MaybeUninit;
 use core::ptr;
 
 #[cfg(feature = "std")]
@@ -32,7 +33,19 @@ pub struct StaticVecIterMut<'a, T: 'a, const N: usize> {
 pub struct StaticVecIntoIter<T, const N: usize> {
   pub(crate) start: usize,
   pub(crate) end: usize,
-  pub(crate) data: StaticVec<T, N>,
+  pub(crate) data: MaybeUninit<[T; N]>,
+}
+
+/// A "draining" iterator, analogous to [`vec::Drain`](alloc::vec::Drain).
+/// Instances of [`StaticVecDrain`](crate::iterators::StaticVecDrain) are created
+/// by the [`drain_iter`](crate::StaticVec::drain_iter) method on [`StaticVec`](crate::StaticVec),
+/// as while the [`drain`](crate::StaticVec::drain) method does have a similar purpose, it works by
+/// immediately returning a new [`StaticVec`](crate::StaticVec) as opposed to an iterator.
+pub struct StaticVecDrain<'a, T: 'a, const N: usize> {
+  pub(crate) start: usize,
+  pub(crate) length: usize,
+  pub(crate) iter: StaticVecIterConst<'a, T, N>,
+  pub(crate) vec: *mut StaticVec<T, N>,
 }
 
 impl<'a, T: 'a, const N: usize> StaticVecIterConst<'a, T, N> {
@@ -66,6 +79,7 @@ impl<'a, T: 'a, const N: usize> StaticVecIterConst<'a, T, N> {
 
 impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
   type Item = &'a T;
+
   #[inline(always)]
   fn next(&mut self) -> Option<Self::Item> {
     match distance_between(self.end, self.start) {
@@ -171,6 +185,7 @@ impl<'a, T: 'a, const N: usize> StaticVecIterMut<'a, T, N> {
 
 impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
   type Item = &'a mut T;
+
   #[inline(always)]
   fn next(&mut self) -> Option<Self::Item> {
     match distance_between(self.end, self.start) {
@@ -247,8 +262,8 @@ impl<T, const N: usize> StaticVecIntoIter<T, N> {
     unsafe {
       format!(
         "Current value of element at `start`: {:?}\nCurrent value of element at `end`: {:?}",
-        &*self.data.ptr_at_unchecked(self.start),
-        &*self.data.ptr_at_unchecked(self.end - 1)
+        &*(self.data.as_ptr() as *const T).add(self.start),
+        &*(self.data.as_ptr() as *const T).add(self.end - 1)
       )
     }
   }
@@ -258,18 +273,21 @@ impl<T, const N: usize> StaticVecIntoIter<T, N> {
   #[inline(always)]
   pub fn as_slice(&self) -> &[T] {
     // Safety: `start` is never null. This function will "at worst" return an empty slice.
-    unsafe { &*ptr::slice_from_raw_parts(self.data.ptr_at_unchecked(self.start), self.len()) }
+    unsafe {
+      &*ptr::slice_from_raw_parts((self.data.as_ptr() as *const T).add(self.start), self.len())
+    }
   }
 }
 
 impl<T, const N: usize> Iterator for StaticVecIntoIter<T, N> {
   type Item = T;
+
   #[inline(always)]
   fn next(&mut self) -> Option<Self::Item> {
     match self.end - self.start {
       0 => None,
       _ => {
-        let res = Some(unsafe { self.data.ptr_at_unchecked(self.start).read() });
+        let res = Some(unsafe { (self.data.as_ptr() as *const T).add(self.start).read() });
         self.start += 1;
         res
       }
@@ -290,7 +308,7 @@ impl<T, const N: usize> DoubleEndedIterator for StaticVecIntoIter<T, N> {
       0 => None,
       _ => {
         self.end -= 1;
-        Some(unsafe { self.data.ptr_at_unchecked(self.end).read() })
+        Some(unsafe { (self.data.as_ptr() as *const T).add(self.end).read() })
       }
     }
   }
@@ -328,10 +346,104 @@ impl<T, const N: usize> Drop for StaticVecIntoIter<T, N> {
       0 => (),
       _ => unsafe {
         ptr::drop_in_place(ptr::slice_from_raw_parts_mut(
-          self.data.mut_ptr_at_unchecked(self.start),
+          (self.data.as_mut_ptr() as *mut T).add(self.start),
           item_count,
         ))
       },
     }
   }
 }
+
+impl<'a, T: 'a, const N: usize> StaticVecDrain<'a, T, N> {
+  /// Returns a string displaying the current values of the
+  /// iterator's `start` and `end` elements on two separate lines.
+  /// Locally requires that `T` implements [Debug](core::fmt::Debug)
+  /// to make it possible to pretty-print the elements.
+  #[cfg(feature = "std")]
+  #[doc(cfg(feature = "std"))]
+  #[inline(always)]
+  pub fn bounds_to_string(&self) -> String
+  where T: Debug {
+    self.iter.bounds_to_string()
+  }
+
+  /// Returns an immutable slice consisting of the current range of elements the iterator has a view
+  /// over.
+  #[inline(always)]
+  pub fn as_slice(&self) -> &[T] {
+    self.iter.as_slice()
+  }
+}
+
+impl<'a, T: 'a, const N: usize> Iterator for StaticVecDrain<'a, T, N> {
+  type Item = T;
+
+  #[inline(always)]
+  fn next(&mut self) -> Option<T> {
+    self
+      .iter
+      .next()
+      .map(|val| unsafe { (val as *const T).read() })
+  }
+
+  #[inline(always)]
+  fn size_hint(&self) -> (usize, Option<usize>) {
+    self.iter.size_hint()
+  }
+}
+
+impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecDrain<'a, T, N> {
+  #[inline(always)]
+  fn next_back(&mut self) -> Option<T> {
+    self
+      .iter
+      .next_back()
+      .map(|val| unsafe { (val as *const T).read() })
+  }
+}
+
+impl<'a, T: 'a, const N: usize> ExactSizeIterator for StaticVecDrain<'a, T, N> {
+  #[inline(always)]
+  fn len(&self) -> usize {
+    self.iter.len()
+  }
+
+  #[inline(always)]
+  fn is_empty(&self) -> bool {
+    self.iter.is_empty()
+  }
+}
+
+impl<'a, T: 'a, const N: usize> FusedIterator for StaticVecDrain<'a, T, N> {}
+unsafe impl<'a, T: 'a, const N: usize> TrustedLen for StaticVecDrain<'a, T, N> {}
+unsafe impl<'a, T: 'a + Sync, const N: usize> Sync for StaticVecDrain<'a, T, N> {}
+unsafe impl<'a, T: 'a + Sync, const N: usize> Send for StaticVecDrain<'a, T, N> {}
+
+impl<'a, T: 'a + Debug, const N: usize> Debug for StaticVecDrain<'a, T, N> {
+  #[inline(always)]
+  fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+    f.debug_tuple("StaticVecDrain")
+      .field(&self.iter.as_slice())
+      .finish()
+  }
+}
+
+impl<'a, T: 'a, const N: usize> Drop for StaticVecDrain<'a, T, N> {
+  #[inline]
+  fn drop(&mut self) {
+    // Read out any remaining contents first.
+    while let Some(_) = self.next() {}
+    if self.length > 0 {
+      unsafe {
+        let vec_ref = &mut *self.vec;
+        let start = vec_ref.length;
+        let tail = self.start;
+        vec_ref
+          .as_ptr()
+          .add(tail)
+          .copy_to(vec_ref.as_mut_ptr().add(start), self.length);
+        vec_ref.set_len(start + self.length);
+      }
+    }
+  }
+}
