@@ -389,12 +389,13 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// in writing to an out-of-bounds memory region.
   #[inline(always)]
   pub unsafe fn push_unchecked(&mut self, value: T) {
+    let length = self.length;
     debug_assert!(
-      self.is_not_full(),
+      length < N,
       "Attempted to unsafely push to a full StaticVec!"
     );
-    self.mut_ptr_at_unchecked(self.length).write(value);
-    self.length += 1;
+    self.mut_ptr_at_unchecked(length).write(value);
+    self.set_len(length + 1);
   }
 
   /// Pops a value from the end of the StaticVec and returns it directly without asserting that
@@ -411,15 +412,16 @@ impl<T, const N: usize> StaticVec<T, N> {
       self.is_not_empty(),
       "Attempted to unsafely pop from an empty StaticVec!"
     );
-    self.length -= 1;
-    self.ptr_at_unchecked(self.length).read()
+    let new_length = self.length - 1;
+    self.set_len(new_length);
+    self.ptr_at_unchecked(new_length).read()
   }
 
   /// Pushes `value` to the StaticVec if its current length is less than its capacity,
   /// or returns a [`PushCapacityError`](crate::errors::PushCapacityError) otherwise.
   #[inline(always)]
   pub fn try_push(&mut self, value: T) -> Result<(), PushCapacityError<T, N>> {
-    if self.length < N {
+    if self.is_not_full() {
       unsafe {
         self.push_unchecked(value);
       };
@@ -433,7 +435,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// full; that is, if `self.len() == self.capacity()`.
   #[inline(always)]
   pub fn push(&mut self, value: T) {
-    assert!(self.length < N);
+    assert!(self.is_not_full());
     unsafe { self.push_unchecked(value) };
   }
 
@@ -498,13 +500,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline]
   pub fn remove(&mut self, index: usize) -> T {
     assert!(index < self.length);
-    unsafe {
-      let p = self.mut_ptr_at_unchecked(index);
-      let res = p.read();
-      p.offset(1).copy_to(p, self.length - index - 1);
-      self.length -= 1;
-      res
-    }
+    unsafe { self.drain(index..=index).as_ptr().read() }
   }
 
   /// Removes the first instance of `item` from the StaticVec if the item exists.
@@ -526,8 +522,9 @@ impl<T, const N: usize> StaticVec<T, N> {
   pub fn swap_pop(&mut self, index: usize) -> Option<T> {
     if index < self.length {
       unsafe {
-        let last_value = self.ptr_at_unchecked(self.length - 1).read();
-        self.length -= 1;
+        let new_length = self.length - 1;
+        let last_value = self.ptr_at_unchecked(new_length).read();
+        self.set_len(new_length);
         Some(self.mut_ptr_at_unchecked(index).replace(last_value))
       }
     } else {
@@ -542,8 +539,9 @@ impl<T, const N: usize> StaticVec<T, N> {
   pub fn swap_remove(&mut self, index: usize) -> T {
     assert!(index < self.length);
     unsafe {
-      let last_value = self.ptr_at_unchecked(self.length - 1).read();
-      self.length -= 1;
+      let new_length = self.length - 1;
+      let last_value = self.ptr_at_unchecked(new_length).read();
+      self.set_len(new_length);
       self.mut_ptr_at_unchecked(index).replace(last_value)
     }
   }
@@ -553,12 +551,13 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// Any values that exist in positions after `index` are shifted to the right.
   #[inline]
   pub fn insert(&mut self, index: usize, value: T) {
-    assert!(self.length < N && index <= self.length);
+    let old_length = self.length;
+    assert!(old_length < N && index <= old_length);
     unsafe {
       let p = self.mut_ptr_at_unchecked(index);
-      p.copy_to(p.offset(1), self.length - index);
+      p.copy_to(p.offset(1), old_length - index);
       p.write(value);
-      self.length += 1;
+      self.set_len(old_length + 1);
     }
   }
 
@@ -575,21 +574,21 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline]
   pub fn insert_many<I: IntoIterator<Item = T>>(&mut self, index: usize, iter: I)
   where I::IntoIter: ExactSizeIterator<Item = T> {
+    let old_length = self.length;
     assert!(
-      self.length < N && index <= self.length,
+      old_length < N && index <= old_length,
       "Insufficient remaining capacity / out of bounds!"
     );
     let mut it = iter.into_iter();
-    if index == self.length {
+    if index == old_length {
       return self.extend(it);
     }
     let iter_size = it.len();
     assert!(
-      index + iter_size >= index && (self.length - index) + iter_size < N,
+      index + iter_size >= index && (old_length - index) + iter_size < N,
       "Insufficient remaining capacity / out of bounds!"
     );
     unsafe {
-      let old_length = self.length;
       let mut this = self.mut_ptr_at_unchecked(index);
       this.copy_to(this.add(iter_size), old_length - index);
       self.length = index;
@@ -617,12 +616,13 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// otherwise. Any values that exist in positions after `index` are shifted to the right.
   #[inline]
   pub fn try_insert(&mut self, index: usize, value: T) -> Result<(), CapacityError<N>> {
-    if self.length < N && index <= self.length {
+    let old_length = self.length;
+    if old_length < N && index <= old_length {
       unsafe {
         let p = self.mut_ptr_at_unchecked(index);
-        p.copy_to(p.offset(1), self.length - index);
+        p.copy_to(p.offset(1), old_length - index);
         p.write(value);
-        self.length += 1;
+        self.set_len(old_length + 1);
         Ok(())
       }
     } else {
@@ -685,6 +685,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline]
   pub fn sorted(&self) -> Self
   where T: Copy + Ord {
+    // StaticVec uses specialization to have an optimized version of `Clone` for copy types.
     let mut res = self.clone();
     res.sort();
     res
@@ -697,6 +698,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline]
   pub fn sorted_unstable(&self) -> Self
   where T: Copy + Ord {
+    // StaticVec uses specialization to have an optimized version of `Clone` for copy types.
     let mut res = self.clone();
     res.sort_unstable();
     res
@@ -709,7 +711,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   pub fn reversed(&self) -> Self
   where T: Copy {
     Self {
-      data: reverse_copy(&self.data),
+      data: reverse_copy(self.length, &self.data),
       length: self.length,
     }
   }
@@ -776,14 +778,15 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline(always)]
   pub fn extend_from_slice(&mut self, other: &[T])
   where T: Copy {
-    let added_length = other.len().min(self.remaining_capacity());
+    let old_length = self.length;
+    let added_length = other.len().min(N - old_length);
     // Safety: added_length is <= our remaining capacity and other.len.
     unsafe {
       other
         .as_ptr()
-        .copy_to_nonoverlapping(self.mut_ptr_at_unchecked(self.length), added_length);
+        .copy_to_nonoverlapping(self.mut_ptr_at_unchecked(old_length), added_length);
+      self.set_len(old_length + added_length);
     }
-    self.length += added_length;
   }
 
   /// Copies and appends all elements, if any, of a slice to the StaticVec if the
@@ -792,16 +795,17 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline(always)]
   pub fn try_extend_from_slice(&mut self, other: &[T]) -> Result<(), CapacityError<N>>
   where T: Copy {
+    let old_length = self.length;
     let added_length = other.len();
-    if self.remaining_capacity() < added_length {
+    if N - old_length < added_length {
       return Err(CapacityError {});
     }
     unsafe {
       other
         .as_ptr()
-        .copy_to_nonoverlapping(self.mut_ptr_at_unchecked(self.length), added_length);
+        .copy_to_nonoverlapping(self.mut_ptr_at_unchecked(old_length), added_length);
+      self.set_len(old_length + added_length);
     }
-    self.length += added_length;
     Ok(())
   }
 
@@ -814,18 +818,19 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// constraint of `self`.)
   #[inline]
   pub fn append<const N2: usize>(&mut self, other: &mut StaticVec<T, N2>) {
-    let item_count = self.remaining_capacity().min(other.length);
+    let old_length = self.length;
+    let item_count = (N - old_length).min(other.length);
     let other_new_length = other.length - item_count;
     unsafe {
       self
-        .mut_ptr_at_unchecked(self.length)
+        .mut_ptr_at_unchecked(old_length)
         .copy_from_nonoverlapping(other.as_ptr(), item_count);
       other
         .as_mut_ptr()
         .copy_from(other.ptr_at_unchecked(item_count), other_new_length);
+      other.set_len(other_new_length);
+      self.set_len(old_length + item_count);
     }
-    other.length = other_new_length;
-    self.length += item_count;
   }
 
   /// Returns a new StaticVec consisting of the elements of `self` and `other` concatenated in
@@ -850,16 +855,18 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline]
   pub fn concat<const N2: usize>(&self, other: &StaticVec<T, N2>) -> StaticVec<T, { N + N2 }>
   where T: Copy {
+    let length = self.length;
+    let other_length = other.length;
     let mut res = StaticVec::new();
     unsafe {
       self
         .as_ptr()
-        .copy_to_nonoverlapping(res.as_mut_ptr(), self.length);
+        .copy_to_nonoverlapping(res.as_mut_ptr(), length);
       other
         .as_ptr()
-        .copy_to_nonoverlapping(res.mut_ptr_at_unchecked(self.length), other.length);
+        .copy_to_nonoverlapping(res.mut_ptr_at_unchecked(length), other_length);
+      res.set_len(length + other_length);
     }
-    res.length = self.length + other.length;
     res
   }
 
@@ -914,7 +921,8 @@ impl<T, const N: usize> StaticVec<T, N> {
     // in return types it seems.
     let mut res_ptr = res.as_mut_ptr() as *mut T;
     let mut i = 0;
-    while i < self.length - 1 {
+    let length = self.length;
+    while i < length - 1 {
       unsafe {
         res_ptr.write(self.ptr_at_unchecked(i).read());
         res_ptr.offset(1).write(separator);
@@ -922,8 +930,11 @@ impl<T, const N: usize> StaticVec<T, N> {
         i += 1
       }
     }
-    unsafe { res_ptr.write(self.ptr_at_unchecked(i).read()) };
-    res.length = (self.length * 2) - 1;
+    unsafe {
+      res_ptr.write(self.ptr_at_unchecked(i).read());
+      res.set_len((length * 2) - 1);
+    }
+
     res
   }
 
@@ -941,12 +952,13 @@ impl<T, const N: usize> StaticVec<T, N> {
       return StaticVec::new();
     }
     let mut res = StaticVec::new();
+    let length = self.length;
     unsafe {
-      for item in self.as_slice().get_unchecked(0..self.length - 1) {
+      for item in self.as_slice().get_unchecked(0..length - 1) {
         res.push_unchecked(item.clone());
         res.push_unchecked(separator.clone());
       }
-      res.push_unchecked(self.get_unchecked(self.length - 1).clone());
+      res.push_unchecked(self.get_unchecked(length - 1).clone());
     }
     res
   }
@@ -992,12 +1004,13 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline(always)]
   pub fn into_vec(mut self) -> Vec<T> {
     let mut res = Vec::with_capacity(N);
+    let length = self.length;
     unsafe {
       self
         .as_ptr()
-        .copy_to_nonoverlapping(res.as_mut_ptr(), self.length);
-      res.set_len(self.length);
-      self.length = 0;
+        .copy_to_nonoverlapping(res.as_mut_ptr(), length);
+      res.set_len(length);
+      self.set_len(0);
       res
     }
   }
@@ -1009,6 +1022,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   // question.
   where R: RangeBounds<usize> {
     // Borrowed this part from normal Vec's implementation.
+    let current_length = self.length;
     let start = match range.start_bound() {
       Included(&idx) => idx,
       Excluded(&idx) => idx + 1,
@@ -1017,9 +1031,9 @@ impl<T, const N: usize> StaticVec<T, N> {
     let end = match range.end_bound() {
       Included(&idx) => idx + 1,
       Excluded(&idx) => idx,
-      Unbounded => self.length,
+      Unbounded => current_length,
     };
-    assert!(start <= end && end <= self.length);
+    assert!(start <= end && end <= current_length);
     let res_length = end - start;
     Self {
       data: {
@@ -1030,8 +1044,8 @@ impl<T, const N: usize> StaticVec<T, N> {
             .copy_to_nonoverlapping(res.as_mut_ptr() as *mut T, res_length);
           self
             .ptr_at_unchecked(end)
-            .copy_to(self.mut_ptr_at_unchecked(start), self.length - end);
-          self.length -= res_length;
+            .copy_to(self.mut_ptr_at_unchecked(start), current_length - end);
+          self.set_len(current_length - res_length);
           res
         }
       },
@@ -1039,6 +1053,42 @@ impl<T, const N: usize> StaticVec<T, N> {
     }
   }
 
+  /// Removes the specified range of elements from the StaticVec and returns them in a
+  /// [`StaticVecDrain`](crate::iterators::StaticVecDrain).
+  #[inline]
+  pub fn drain_iter<R>(&mut self, range: R) -> StaticVecDrain<T, N>
+  where R: RangeBounds<usize> {
+    // Borrowed this part from normal Vec's implementation.
+    let length = self.length;
+    let start = match range.start_bound() {
+      Included(&idx) => idx,
+      Excluded(&idx) => idx + 1,
+      Unbounded => 0,
+    };
+    let end = match range.end_bound() {
+      Included(&idx) => idx + 1,
+      Excluded(&idx) => idx,
+      Unbounded => length,
+    };
+    assert!(start <= end && end <= length);
+    unsafe {
+      // Set the length to 0 to avoid memory issues if anything goes wrong with
+      // the Drain.
+      self.set_len(start);
+      // Create the StaticVecDrain from the specified range.
+      StaticVecDrain {
+        start: end,
+        length: length - end,
+        iter: StaticVecIterConst {
+          start: self.mut_ptr_at_unchecked(start),
+          end: self.mut_ptr_at_unchecked(end),
+          marker: PhantomData,
+        },
+        vec: self,
+      }
+    }
+  }
+
   /// Removes all elements in the StaticVec for which `filter` returns true and
   /// returns them in a new one.
   #[inline]
@@ -1077,8 +1127,8 @@ impl<T, const N: usize> StaticVec<T, N> {
   pub fn truncate(&mut self, length: usize) {
     if length < self.length {
       let old_length = self.length;
-      self.length = length;
       unsafe {
+        self.set_len(length);
         ptr::drop_in_place(self.as_mut_slice().get_unchecked_mut(length..old_length));
       }
     }
@@ -1089,11 +1139,12 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// and the new one will contain elements `at..length`.
   #[inline]
   pub fn split_off(&mut self, at: usize) -> Self {
-    assert!(at <= self.length);
-    let split_length = self.length - at;
-    self.length = at;
+    let length = self.length;
+    assert!(at <= length);
+    let split_length = length - at;
     Self {
       data: unsafe {
+        self.set_len(at);
         let mut split = Self::new_data_uninit();
         self
           .ptr_at_unchecked(at)
@@ -1284,7 +1335,8 @@ impl<T, const N: usize> StaticVec<T, N> {
       res.dedup();
       res
     } else {
-      let mut res = StaticVec::from_iter(other.iter().chain(self.difference(other).iter()).cloned());
+      let mut res =
+        StaticVec::from_iter(other.iter().chain(self.difference(other).iter()).cloned());
       res.dedup();
       res
     }
