@@ -374,6 +374,53 @@ fn drain() {
   assert_eq!(v, &[]);
 }
 
+#[test]
+fn drain_iter() {
+  let mut v = StaticVec::from([0; 8]);
+  v.pop();
+  v.drain_iter(0..7);
+  assert_eq!(&v[..], &[]);
+  v.extend(0..);
+  v.drain_iter(1..4);
+  assert_eq!(&v[..], &[0, 4, 5, 6, 7]);
+  let u: StaticVec<_, 3> = v.drain_iter(1..4).rev().collect();
+  assert_eq!(&u[..], &[6, 5, 4]);
+  assert_eq!(&v[..], &[0, 7]);
+  v.drain_iter(..);
+  assert_eq!(&v[..], &[]);
+  let mut v2 = StaticVec::from([0; 8]);
+  v2.drain_iter(0..=7);
+  assert_eq!(&v2[..], &[]);
+  v2.extend(0..);
+  v2.drain_iter(1..=4);
+  assert_eq!(&v2[..], &[0, 5, 6, 7]);
+  let u: StaticVec<_, 3> = v2.drain_iter(1..=2).rev().collect();
+  assert_eq!(&u[..], &[6, 5]);
+  assert_eq!(&v2[..], &[0, 7]);
+  v2.drain_iter(..);
+  assert_eq!(&v2[..], &[]);
+  let mut v3 = staticvec![
+    Box::new(12),
+    Box::new(12),
+    Box::new(12),
+    Box::new(12),
+    Box::new(12),
+    Box::new(12),
+    Box::new(12),
+    Box::new(12)
+  ];
+  v3.pop();
+  v3.drain_iter(0..7);
+  assert_eq!(&v3[..], &[]);
+}
+
+#[test]
+#[should_panic]
+fn drain_iter_panic() {
+  let mut v3 = StaticVec::from([0; 0]);
+  v3.drain_iter(0..=0);
+}
+
 #[test]
 fn drain_filter() {
   let mut numbers = staticvec![1, 2, 3, 4, 5, 6, 8, 9, 11, 13, 14, 15];
@@ -673,6 +720,24 @@ fn into_iter() {
   assert_eq!("[3]", format!("{:?}", i));
   assert_eq!(i.next().unwrap(), 3);
   assert_eq!("[]", format!("{:?}", i));
+  let v2 = staticvec![
+    Box::new(Struct { s: "AAA" }),
+    Box::new(Struct { s: "BBB" }),
+    Box::new(Struct { s: "CCC" })
+  ];
+  let mut i2 = v2.into_iter();
+  assert_eq!(i2.next().unwrap(), Box::new(Struct { s: "AAA" }));
+  assert_eq!(i2.next().unwrap(), Box::new(Struct { s: "BBB" }));
+  assert_eq!(i2.next().unwrap(), Box::new(Struct { s: "CCC" }));
+  assert_eq!("[]", format!("{:?}", i2));
+  let v3 = staticvec![
+    Box::new(Struct { s: "AAA" }),
+    Box::new(Struct { s: "BBB" }),
+    Box::new(Struct { s: "CCC" })
+  ];
+  let mut i3 = v3.into_iter();
+  // We do this so Miri can make sure it drops the remaining values properly.
+  i3.next();
 }
 
 #[cfg(feature = "std")]
@@ -1006,6 +1071,17 @@ fn retain() {
 fn reversed() {
   let v = staticvec![1, 2, 3].reversed();
   assert!(v == [3, 2, 1]);
+  let mut x = StaticVec::<f64, 24>::new();
+  let mut y = StaticVec::<f64, 12>::new();
+  for _ in 0..12 {
+    y.push(12.0);
+  }
+  x.append(&mut y);
+  assert_eq!(x.reversed().len(), 12);
+  assert_eq!(
+    x.reversed(),
+    [12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0]
+  );
 }
 
 #[test]
